/*
 * Copyright (c) 2011 Samsung Electronics Co., Ltd All Rights Reserved
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

#include "main.h"

/*********************************************************
  button
 ********************************************************/
static void _default_cb(void *data, Evas_Object * obj, void *event_info);
static void _color_cb(void *data, Evas_Object * obj, void *event_info);
static void _icon_cb(void *data, Evas_Object * obj, void *event_info);
static void _nextdepth_cb(void *data, Evas_Object * obj, void *event_info);

static struct _menu_item _menu_its[] = {
	{"Default button", _default_cb},
	{"Color button", _color_cb},
	{"Icon button", _icon_cb},
	{"Nextdepth button", _nextdepth_cb},
	/* do not delete below */
	{NULL, NULL}
};

static void _list_click(void *data, Evas_Object * obj, void *event_info)
{
	Elm_Object_Item *it = (Elm_Object_Item *) elm_list_selected_item_get(obj);

	if (!it) {
		fprintf(stderr, "list item is NULL\n");
		return;
	}

	elm_list_item_selected_set(it, EINA_FALSE);
}

static void _clicked_cb(void *data, Evas_Object * obj, void *event_info)
{
	printf("clicked\n");
}

static void _pressed_cb(void *data, Evas_Object * obj, void *event_info)
{
	printf("pressed\n");
}

static void _unpressed_cb(void *data, Evas_Object * obj, void *event_info)
{
	printf("unpressed\n");
}

static Evas_Object *_create_list_winset(struct appdata *ad)
{
	Evas_Object *li;
	struct _menu_item *menu_its;
	int idx = 0;

	if (!ad) return NULL;

	li = elm_list_add(ad->nf);
	elm_list_mode_set(li, ELM_LIST_COMPRESS);
	evas_object_smart_callback_add(li, "selected", _list_click, NULL);

	menu_its = _menu_its;
	while (menu_its[idx].name != NULL) {
		elm_list_item_append(li, menu_its[idx].name, NULL, NULL, menu_its[idx].func, ad);
		++idx;
	}
	elm_list_go(li);

	return li;
}

static Evas_Object *_create_scroller(Evas_Object * parent)
{
	Evas_Object *scroller = elm_scroller_add(parent);

	elm_scroller_bounce_set(scroller, EINA_FALSE, EINA_TRUE);
	elm_scroller_policy_set(scroller, ELM_SCROLLER_POLICY_OFF, ELM_SCROLLER_POLICY_AUTO);
	evas_object_show(scroller);

	return scroller;
}

static void _default_cb(void *data, Evas_Object * obj, void *event_info)
{
	Evas_Object *scroller;
	Evas_Object *layout, *btn;
	Elm_Object_Item *it;
	struct appdata *ad = (struct appdata *)data;

	if (!ad) return;

	scroller = _create_scroller(ad->nf);
	evas_object_data_set(scroller, "appdata", ad);

	layout = elm_layout_add(scroller);
	elm_layout_file_set(layout, ELM_DEMO_EDJ, "elmdemo-test/button/normal");
	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

	btn = elm_button_add(layout);
	elm_object_text_set(btn, _("Button Example<br> This is Normal"));
	evas_object_size_hint_weight_set(btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_part_content_set(layout, "btn1", btn);
	evas_object_smart_callback_add(btn, "clicked", _clicked_cb, ad->nf);
	evas_object_smart_callback_add(btn, "unpressed", _unpressed_cb, NULL);
	evas_object_smart_callback_add(btn, "pressed", _pressed_cb, NULL);

	btn = elm_button_add(layout);
	elm_object_text_set(btn, _("Button Example<br> This is Disabled"));
	evas_object_size_hint_weight_set(btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_part_content_set(layout, "btn2", btn);
	elm_object_disabled_set(btn, EINA_TRUE);

	btn = elm_button_add(layout);
	elm_object_text_set(btn, _("Button1"));
	evas_object_size_hint_weight_set(btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_part_content_set(layout, "btn5", btn);
	evas_object_smart_callback_add(btn, "clicked", _clicked_cb, ad->nf);
	evas_object_smart_callback_add(btn, "unpressed", _unpressed_cb, NULL);
	evas_object_smart_callback_add(btn, "pressed", _pressed_cb, NULL);

	btn = elm_button_add(layout);
	elm_object_style_set(btn, "transparent");
	elm_object_text_set(btn, _("Button2"));
	evas_object_size_hint_weight_set(btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

	Evas_Object *ic = elm_image_add(btn);
	elm_image_file_set(ic, ICON_DIR "/logo_small.png", NULL);
	evas_object_size_hint_min_set(ic, 60 * elm_config_scale_get(), 60 * elm_config_scale_get());
	evas_object_size_hint_max_set(ic, 60 * elm_config_scale_get(), 60 * elm_config_scale_get());
	elm_object_content_set(btn, ic);
	elm_object_part_content_set(layout, "btn6", btn);
	evas_object_smart_callback_add(btn, "clicked", _clicked_cb, NULL);
	evas_object_smart_callback_add(btn, "unpressed", _unpressed_cb, NULL);
	evas_object_smart_callback_add(btn, "pressed", _pressed_cb, NULL);

	elm_object_content_set(scroller, layout);

	it = elm_naviframe_item_push(ad->nf, NULL, NULL, NULL, scroller, NULL);
	elm_naviframe_item_title_enabled_set(it, EINA_FALSE, EINA_FALSE);
}

static void _color_cb(void *data, Evas_Object * obj, void *event_info)
{
	Evas_Object *scroller;
	Evas_Object *layout, *btn;
	Elm_Object_Item *it;
	Evas_Object *ic;
	struct appdata *ad = (struct appdata *)data;

	if (!ad) return;

	scroller = _create_scroller(ad->nf);
	evas_object_data_set(scroller, "appdata", ad);

	layout = elm_layout_add(scroller);
	elm_layout_file_set(layout, ELM_DEMO_EDJ, "elmdemo-test/button/color");
	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

	btn = elm_button_add(layout);
	elm_object_style_set(btn, "green");
	elm_object_text_set(btn, _("Button1"));
	evas_object_size_hint_weight_set(btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_part_content_set(layout, "btn1", btn);
	evas_object_smart_callback_add(btn, "clicked", _clicked_cb, NULL);
	evas_object_smart_callback_add(btn, "unpressed", _unpressed_cb, NULL);
	evas_object_smart_callback_add(btn, "pressed", _pressed_cb, NULL);

	btn = elm_button_add(layout);
	elm_object_style_set(btn, "orange");
	elm_object_text_set(btn, _("Button2"));
	evas_object_size_hint_weight_set(btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_part_content_set(layout, "btn2", btn);
	evas_object_smart_callback_add(btn, "clicked", _clicked_cb, NULL);
	evas_object_smart_callback_add(btn, "unpressed", _unpressed_cb, NULL);
	evas_object_smart_callback_add(btn, "pressed", _pressed_cb, NULL);

	btn = elm_button_add(layout);
	elm_object_style_set(btn, "red");
	elm_object_text_set(btn, _("Button3"));
	evas_object_size_hint_weight_set(btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_part_content_set(layout, "btn3", btn);
	evas_object_smart_callback_add(btn, "clicked", _clicked_cb, NULL);
	evas_object_smart_callback_add(btn, "unpressed", _unpressed_cb, NULL);
	evas_object_smart_callback_add(btn, "pressed", _pressed_cb, NULL);

	btn = elm_button_add(layout);
	elm_object_text_set(btn, _("Button4"));
	ic = elm_image_add(btn);
	elm_image_file_set(ic, ICON_DIR "/logo_small.png", NULL);
	elm_object_content_set(btn, ic);
	evas_object_size_hint_weight_set(btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_part_content_set(layout, "btn4", btn);
	evas_object_smart_callback_add(btn, "clicked", _clicked_cb, NULL);
	evas_object_smart_callback_add(btn, "unpressed", _unpressed_cb, NULL);
	evas_object_smart_callback_add(btn, "pressed", _pressed_cb, NULL);

	elm_object_content_set(scroller, layout);

	it = elm_naviframe_item_push(ad->nf, NULL, NULL, NULL, scroller, NULL);
	elm_naviframe_item_title_enabled_set(it, EINA_FALSE, EINA_FALSE);
}

static void _icon_cb(void *data, Evas_Object * obj, void *event_info)
{
	Evas_Object *scroller;
	Evas_Object *layout, *btn, *ic;
	Elm_Object_Item *it;
	struct appdata *ad = (struct appdata *)data;

	if (!ad) return;

	scroller = _create_scroller(ad->nf);
	evas_object_data_set(scroller, "appdata", ad);

	layout = elm_layout_add(scroller);
	elm_layout_file_set(layout, ELM_DEMO_EDJ, "elmdemo-test/button/icon");
	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

	btn = elm_button_add(layout);
	ic = elm_image_add(btn);
	elm_image_file_set(ic, ICON_DIR "/logo_small.png", NULL);
	evas_object_size_hint_min_set(ic, 60 * elm_config_scale_get(), 60 * elm_config_scale_get());
	evas_object_size_hint_max_set(ic, 60 * elm_config_scale_get(), 60 * elm_config_scale_get());
	elm_object_content_set(btn, ic);
	evas_object_smart_callback_add(btn, "clicked", _clicked_cb, NULL);
	evas_object_smart_callback_add(btn, "unpressed", _unpressed_cb, NULL);
	evas_object_smart_callback_add(btn, "pressed", _pressed_cb, NULL);
	elm_object_part_content_set(layout, "btn1", btn);

	btn = elm_button_add(layout);
	ic = elm_image_add(btn);
	elm_image_resizable_set(ic, EINA_TRUE, EINA_TRUE);
	elm_image_file_set(ic, ICON_DIR "/logo_small.png", NULL);
	evas_object_size_hint_min_set(ic, 60 * elm_config_scale_get(), 60 * elm_config_scale_get());
	evas_object_size_hint_max_set(ic, 60 * elm_config_scale_get(), 60 * elm_config_scale_get());
	elm_object_disabled_set(btn, EINA_TRUE);
	elm_object_content_set(btn, ic);
	evas_object_size_hint_weight_set(btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_part_content_set(layout, "btn2", btn);
	evas_object_smart_callback_add(btn, "clicked", _clicked_cb, NULL);
	evas_object_smart_callback_add(btn, "unpressed", _unpressed_cb, NULL);
	evas_object_smart_callback_add(btn, "pressed", _pressed_cb, NULL);

	btn = elm_button_add(layout);
	ic = elm_image_add(btn);
	elm_image_resizable_set(ic, 1, 1);
	elm_image_file_set(ic, ICON_DIR "/logo_small.png", NULL);
	evas_object_size_hint_min_set(ic, 60 * elm_config_scale_get(), 60 * elm_config_scale_get());
	evas_object_size_hint_max_set(ic, 60 * elm_config_scale_get(), 60 * elm_config_scale_get());
	elm_object_content_set(btn, ic);
	evas_object_size_hint_weight_set(btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_part_content_set(layout, "btn3", btn);
	evas_object_smart_callback_add(btn, "clicked", _clicked_cb, NULL);
	evas_object_smart_callback_add(btn, "unpressed", _unpressed_cb, NULL);
	evas_object_smart_callback_add(btn, "pressed", _pressed_cb, NULL);
	elm_object_content_set(scroller, layout);

	it = elm_naviframe_item_push(ad->nf, NULL, NULL, NULL, scroller, NULL);
	elm_naviframe_item_title_enabled_set(it, EINA_FALSE, EINA_FALSE);
}

static void _nextdepth_cb(void *data, Evas_Object * obj, void *event_info)
{
	Evas_Object *scroller;
	Evas_Object *layout, *btn;
	Elm_Object_Item *it;
	struct appdata *ad = (struct appdata *)data;

	if (!ad) return;

	scroller = _create_scroller(ad->nf);
	evas_object_data_set(scroller, "appdata", ad);

	layout = elm_layout_add(scroller);
	elm_layout_file_set(layout, ELM_DEMO_EDJ, "elmdemo-test/button/nextdepth");
	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

	btn = elm_button_add(layout);
	elm_object_style_set(btn, "nextdepth");
	evas_object_size_hint_weight_set(btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_smart_callback_add(btn, "clicked", _clicked_cb, NULL);
	evas_object_smart_callback_add(btn, "unpressed", _unpressed_cb, NULL);
	evas_object_smart_callback_add(btn, "pressed", _pressed_cb, NULL);
	elm_object_part_content_set(layout, "btn", btn);

	elm_object_content_set(scroller, layout);

	it = elm_naviframe_item_push(ad->nf, NULL, NULL, NULL, scroller, NULL);
	elm_naviframe_item_title_enabled_set(it, EINA_FALSE, EINA_FALSE);
}

void button_cb(void *data, Evas_Object * obj, void *event_info)
{
	struct appdata *ad;
	Evas_Object *list;

	ad = (struct appdata *)data;
	if (!ad) return;

	list = _create_list_winset(ad);
	elm_naviframe_item_push(ad->nf, _("Button"), NULL, NULL, list, NULL);
}
