/*
 * Copyright (c) 2011 Samsung Electronics Co., Ltd All Rights Reserved
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

#include "main.h"

/*********************************************************
 contextpopup
 ********************************************************/
static Evas_Coord touch_x;
static Evas_Coord touch_y;
static Evas_Object* ctxpopup = NULL;
struct appdata *ad;

typedef struct _Ctxpopup_Data
{
	Evas_Object *ctx;
	Evas_Object *btn;
} Ctxpopup_Data;
Ctxpopup_Data ctx_data;

static Eina_Bool _pop_cb(void *data, Elm_Object_Item *it)
{
	if(ctxpopup) {
		evas_object_del(ctxpopup);
		ctxpopup = NULL;
	}

	elm_theme_extension_del(NULL, ELM_DEMO_EDJ);
	ecore_event_handler_del(data);

	return EINA_TRUE;
}

static void _move_ctxpopup(Evas_Object *ctxpopup, Evas_Object *btn)
{
   Evas_Coord x, y, w , h;
   evas_object_geometry_get(btn, &x, &y, &w, &h);
   evas_object_move(ctxpopup, x + (w / 2), y + (h /2));
}

static void _dismissed_cb(void *data, Evas_Object *obj, void *event_info)
{
	evas_object_smart_callback_del(ctxpopup,"dismissed", _dismissed_cb);
	evas_object_del(ctxpopup);
	ctxpopup = NULL;
}

static void _ctxpopup_cb(void *data, Evas_Object *obj, void *event_info)
{
	const char *label = elm_object_item_text_get((Elm_Object_Item *) event_info);
	if(label) fprintf(stderr, "text(%s) is clicked\n", label);

	Evas_Object *icon = elm_object_item_content_get((Elm_Object_Item *) event_info);
	if(icon) fprintf(stderr, "icon is clicked\n");
}

/*********************************************************
5 Icons
 ********************************************************/
static void _create_ctxpopup_5icon( void *data, Evas_Object *obj, void *event_info )
{
	char buf[255];
	Evas_Object* icon;

	if(ctxpopup) {
		evas_object_del(ctxpopup);
		ctxpopup = NULL;
	}

	ctxpopup = elm_ctxpopup_add(ad->nf);
	eext_object_event_callback_add(ctxpopup, EEXT_CALLBACK_BACK, eext_ctxpopup_back_cb, NULL);
	evas_object_smart_callback_add(ctxpopup,"dismissed", _dismissed_cb, NULL);
	elm_ctxpopup_horizontal_set(ctxpopup, EINA_TRUE);

	//Icon 1
	icon = elm_image_add(ctxpopup);
	snprintf(buf, sizeof(buf), "%s/b_option_icon_contacts.png", ICON_DIR);
	elm_image_file_set(icon, buf, NULL);
	elm_ctxpopup_item_append(ctxpopup, NULL, icon, _ctxpopup_cb, ctxpopup);

	//Icon 2
	icon = elm_image_add(ctxpopup);
	snprintf(buf, sizeof(buf), "%s/b_option_icon_call.png", ICON_DIR);
	elm_image_file_set(icon, buf, NULL);
	elm_ctxpopup_item_append(ctxpopup, NULL, icon, _ctxpopup_cb, ctxpopup);

	//Icon 3
	icon = elm_image_add(ctxpopup);
	snprintf(buf, sizeof(buf), "%s/b_option_icon_message.png", ICON_DIR);
	elm_image_file_set(icon, buf, NULL);
	elm_ctxpopup_item_append(ctxpopup, NULL, icon, _ctxpopup_cb, ctxpopup);

	//Icon 4
	icon = elm_image_add(ctxpopup);
	snprintf(buf, sizeof(buf), "%s/b_option_icon_contacts.png", ICON_DIR);
	elm_image_file_set(icon, buf, NULL);
	elm_ctxpopup_item_append(ctxpopup, NULL, icon, _ctxpopup_cb, ctxpopup);

	//Icon 5
	icon = elm_image_add(ctxpopup);
	snprintf(buf, sizeof(buf), "%s/b_option_icon_call.png", ICON_DIR);
	elm_image_file_set(icon, buf, NULL);
	elm_ctxpopup_item_append(ctxpopup, NULL, icon, _ctxpopup_cb, ctxpopup);

	_move_ctxpopup(ctxpopup, obj);
	evas_object_show(ctxpopup);
}
/*********************************************************
5 Icons End
 ********************************************************/

/*********************************************************
2 Text & 2 Icons
 ********************************************************/
static void _create_ctxpopup_2text_2icon( void *data, Evas_Object *obj, void *event_info )
{
	char buf[255];
	Evas_Object* icon;

	if(ctxpopup) {
		evas_object_del(ctxpopup);
		ctxpopup = NULL;
	}

	ctxpopup = elm_ctxpopup_add(ad->nf);
	//ea_object_event_callback_add(ctxpopup, EA_CALLBACK_BACK, ea_ctxpopup_back_cb, NULL);
	eext_object_event_callback_add(ctxpopup, EEXT_CALLBACK_BACK, eext_ctxpopup_back_cb, NULL);
	evas_object_smart_callback_add(ctxpopup,"dismissed", _dismissed_cb, NULL);
	elm_ctxpopup_horizontal_set(ctxpopup, EINA_TRUE);

	elm_ctxpopup_item_append(ctxpopup, _("Function"), NULL, _ctxpopup_cb, ctxpopup);
	elm_ctxpopup_item_append(ctxpopup, _("Text"), NULL, _ctxpopup_cb, ctxpopup);

	//Icon 1
	icon = elm_image_add(ctxpopup);
	snprintf(buf, sizeof(buf), "%s/b_option_icon_contacts.png", ICON_DIR);
	elm_image_file_set(icon, buf, NULL);
	elm_ctxpopup_item_append(ctxpopup, NULL, icon, _ctxpopup_cb, ctxpopup);

	//Icon 2
	icon = elm_image_add(ctxpopup);
	snprintf(buf, sizeof(buf), "%s/b_option_icon_call.png", ICON_DIR);
	elm_image_file_set(icon, buf, NULL);
	elm_ctxpopup_item_append(ctxpopup, NULL, icon, _ctxpopup_cb, ctxpopup);

	_move_ctxpopup(ctxpopup, obj);
	evas_object_show(ctxpopup);
}
/*********************************************************
2 Text & 2 Icons End
 ********************************************************/

/*********************************************************
Text Only
 ********************************************************/
static void _create_ctxpopup_text_only(void *data, Evas_Object *obj, void *event_info)
{
	if(ctxpopup) {
		evas_object_del(ctxpopup);
		ctxpopup = NULL;
	}

	ctxpopup = elm_ctxpopup_add(ad->nf);
	eext_object_event_callback_add(ctxpopup, EEXT_CALLBACK_BACK, eext_ctxpopup_back_cb, NULL);
	evas_object_smart_callback_add(ctxpopup,"dismissed", _dismissed_cb, NULL);

	elm_ctxpopup_item_append(ctxpopup, _("Option 1"), NULL, _ctxpopup_cb, ctxpopup);
	elm_ctxpopup_item_append(ctxpopup, _("Option 2"), NULL, _ctxpopup_cb, ctxpopup);
	elm_ctxpopup_item_append(ctxpopup, _("Option 3"), NULL, _ctxpopup_cb, ctxpopup);
	elm_ctxpopup_item_append(ctxpopup, _("Option 4"), NULL, _ctxpopup_cb, ctxpopup);
	elm_ctxpopup_item_append(ctxpopup, _("Option 5"), NULL, _ctxpopup_cb, ctxpopup);

	_move_ctxpopup(ctxpopup, obj);
	evas_object_show(ctxpopup);
}
/*********************************************************
Text Only End
 ********************************************************/

/*********************************************************
Icon + Text
 ********************************************************/
static void _create_ctxpopup_icon_text(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *icon;
	char buf[256];

	if(ctxpopup) {
		evas_object_del(ctxpopup);
		ctxpopup = NULL;
	}

	ctxpopup = elm_ctxpopup_add(ad->nf);
	//ea_object_event_callback_add(ctxpopup, EA_CALLBACK_BACK, ea_ctxpopup_back_cb, NULL);
	eext_object_event_callback_add(ctxpopup, EEXT_CALLBACK_BACK, eext_ctxpopup_back_cb, NULL);
	evas_object_smart_callback_add(ctxpopup,"dismissed", _dismissed_cb, NULL);

	icon = elm_image_add(ctxpopup);
	snprintf(buf, sizeof(buf), "%s/b_option_list_icon_action.png", ICON_DIR);
	elm_image_file_set(icon, buf, NULL);
	elm_ctxpopup_item_append(ctxpopup, _("Option 1"), icon, _ctxpopup_cb, ctxpopup);

	icon = elm_image_add(ctxpopup);
	snprintf(buf, sizeof(buf), "%s/b_option_list_icon_delete.png", ICON_DIR);
	elm_image_file_set(icon, buf, NULL);
	elm_ctxpopup_item_append(ctxpopup, _("Option 2"), icon, _ctxpopup_cb, ctxpopup);

	icon = elm_image_add(ctxpopup);
	snprintf(buf, sizeof(buf), "%s/b_option_list_icon_enter.png", ICON_DIR);
	elm_image_file_set(icon, buf, NULL);
	elm_ctxpopup_item_append(ctxpopup, _("Option 3"), icon, _ctxpopup_cb, ctxpopup);

	icon = elm_image_add(ctxpopup);
	snprintf(buf, sizeof(buf), "%s/b_option_list_icon_share.png", ICON_DIR);
	elm_image_file_set(icon, buf, NULL);
	elm_ctxpopup_item_append(ctxpopup, _("Option 4"), icon, _ctxpopup_cb, ctxpopup);

	icon = elm_image_add(ctxpopup);
	snprintf(buf, sizeof(buf), "%s/b_option_list_icon_action.png", ICON_DIR);
	elm_image_file_set(icon, buf, NULL);
	elm_ctxpopup_item_append(ctxpopup, _("Option 5"), icon, _ctxpopup_cb, ctxpopup);

	_move_ctxpopup(ctxpopup, obj);
	evas_object_show(ctxpopup);
}
/*********************************************************
Icon + Text End
 ********************************************************/

static Eina_Bool _mousedown_cb(void *data, int type, void *event)
{
	Ecore_Event_Mouse_Button *ev = event;
	touch_x = ev->x;
	touch_y = ev->y;
	return ECORE_CALLBACK_CANCEL;
}

void ctxpopup_cb(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *layout;
	Evas_Object *btn;
	Ecore_Event_Handler* event;
	Evas_Object *scroller;
	Elm_Object_Item *navi_it;

	ad = (struct appdata*) data;
	if (ad == NULL) return;

	elm_theme_extension_add(NULL, ELM_DEMO_EDJ);
	event = ecore_event_handler_add(ECORE_EVENT_MOUSE_BUTTON_DOWN, _mousedown_cb, NULL);

	//scroller
	scroller = elm_scroller_add(ad->nf);
	elm_scroller_bounce_set(scroller, EINA_FALSE, EINA_FALSE);
	elm_scroller_policy_set(scroller,ELM_SCROLLER_POLICY_OFF,ELM_SCROLLER_POLICY_AUTO);

	//layout
	layout = elm_layout_add(ad->nf);
	elm_layout_file_set(layout, ELM_DEMO_EDJ, "ctxpopup_layout");
	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	navi_it = elm_naviframe_item_push(ad->nf, _("CtxPopup"), NULL, NULL, scroller, NULL);
	elm_naviframe_item_pop_cb_set(navi_it, _pop_cb, event);

	elm_object_content_set(scroller, layout);

	//Text Only
	btn = elm_button_add(layout);
	elm_object_text_set(btn, _("Text Only"));
	evas_object_smart_callback_add(btn, "clicked", _create_ctxpopup_text_only, ad);
	elm_object_part_content_set(layout, "btn1", btn);

	//5 Icon
	btn = elm_button_add(layout);
	elm_object_text_set(btn, _("5 Icon"));
	evas_object_smart_callback_add(btn, "clicked", _create_ctxpopup_5icon, ad);
	elm_object_part_content_set(layout, "btn2", btn);

	//2 Text 2 Icon
	btn = elm_button_add(layout);
	elm_object_text_set(btn, _("2 Text 2 Icon"));
	evas_object_smart_callback_add(btn, "clicked", _create_ctxpopup_2text_2icon, ad);
	elm_object_part_content_set(layout, "btn3", btn);

	//Icon + Text
	btn = elm_button_add(layout);
	elm_object_text_set(btn, _("Icon Text"));
	evas_object_smart_callback_add(btn, "clicked", _create_ctxpopup_icon_text, ad);
	elm_object_part_content_set(layout, "btn4", btn);
}
