/*
 * Copyright (c) 2011 Samsung Electronics Co., Ltd All Rights Reserved
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

#include "main.h"

#define FORMAT "%d/%b/%Y%I:%M%p"

static void _datetime_date_cb(void *data, Evas_Object *obj, void *event_info);
static void _datetime_time_cb(void *data, Evas_Object *obj, void *event_info);

static struct _menu_item _menu_its[] = {
	{"Set date", _datetime_date_cb},
	{"Set time", _datetime_time_cb},
	{NULL, NULL}
};

static Evas_Object* _create_scroller(Evas_Object* parent)
{
	Evas_Object* scroller = elm_scroller_add(parent);
	elm_scroller_bounce_set(scroller, EINA_FALSE, EINA_TRUE);
	elm_scroller_policy_set(scroller,ELM_SCROLLER_POLICY_OFF,ELM_SCROLLER_POLICY_AUTO);
	evas_object_show(scroller);
	return scroller;
}

static void _list_click(void *data, Evas_Object * obj, void *event_info)
{
	Elm_Object_Item *it = (Elm_Object_Item *) elm_list_selected_item_get(obj);

	if (!it) {
		fprintf(stderr, "list item is NULL\n");
		return;
	}

	elm_list_item_selected_set(it, EINA_FALSE);
}

static void _set_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	printf("Set clicked : [TODO] Apply datetime value to system!\n");
	if (!data) return;
	elm_naviframe_item_pop(data);
}

static void _cancel_clicked_cb(void *data, Evas_Object * obj, void *event_info)
{
	printf("Cancel clicked\n");
	if (!data) return;
	elm_naviframe_item_pop(data);
}

static void _datetime_date_cb(void *data, Evas_Object * obj, void *event_info)
{
	Evas_Object *layout, *btn, *scroller;
	Evas_Object *datetime;
	Elm_Object_Item *it;
	struct appdata *ad = (struct appdata *)data;

	if (!ad) return;

	scroller = _create_scroller(ad->nf);

	layout = elm_layout_add(scroller);
	elm_layout_file_set(layout, ELM_DEMO_EDJ, "elmdemo-test/datetime");
	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

	datetime = elm_datetime_add(layout);
	elm_object_style_set(datetime, "datepicker_layout");
	elm_object_part_content_set(layout, "content", datetime);

	btn = elm_button_add(layout);
	elm_object_text_set(btn, _("Cancel"));
	evas_object_size_hint_weight_set(btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_part_content_set(layout, "btn1", btn);
	evas_object_smart_callback_add(btn, "clicked", _cancel_clicked_cb, ad->nf);

	btn = elm_button_add(layout);
	elm_object_text_set(btn, _("Set"));
	evas_object_size_hint_weight_set(btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_part_content_set(layout, "btn2", btn);
	evas_object_smart_callback_add(btn, "clicked", _set_clicked_cb, ad->nf);

	elm_object_content_set(scroller, layout);
	it = elm_naviframe_item_push(ad->nf, "Title", NULL, NULL, scroller, "empty");
}

static void _datetime_time_cb(void *data, Evas_Object * obj, void *event_info)
{
	Evas_Object *layout, *btn, *scroller;
	Evas_Object *datetime;
	Elm_Object_Item *it;
	struct appdata *ad = (struct appdata *)data;

	if (!ad) return;
	scroller = _create_scroller(ad->nf);

	layout = elm_layout_add(scroller);
	elm_layout_file_set(layout, ELM_DEMO_EDJ, "elmdemo-test/datetime");
	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

	datetime = elm_datetime_add(layout);
	elm_object_style_set(datetime, "timepicker_layout");
	elm_object_part_content_set(layout, "content", datetime);
	elm_datetime_format_set(datetime, FORMAT);

	btn = elm_button_add(layout);
	elm_object_text_set(btn, _("Cancel"));
	evas_object_size_hint_weight_set(btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_part_content_set(layout, "btn1", btn);
	evas_object_smart_callback_add(btn, "clicked", _cancel_clicked_cb, ad->nf);

	btn = elm_button_add(layout);
	elm_object_text_set(btn, _("Set"));
	evas_object_size_hint_weight_set(btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_part_content_set(layout, "btn2", btn);
	evas_object_smart_callback_add(btn, "clicked", _set_clicked_cb, ad->nf);
	elm_object_content_set(scroller, layout);
	it = elm_naviframe_item_push(ad->nf, "Title", NULL, NULL, scroller, "empty");
}

static Evas_Object *_create_list_winset(struct appdata *ad)
{
	Evas_Object *li;
	struct _menu_item *menu_its;
	int idx = 0;

	if (!ad) return NULL;

	li = elm_list_add(ad->nf);
	elm_list_mode_set(li, ELM_LIST_COMPRESS);
	evas_object_smart_callback_add(li, "selected", _list_click, NULL);

	menu_its = _menu_its;
	while (menu_its[idx].name != NULL) {
		elm_list_item_append(li, menu_its[idx].name, NULL, NULL, menu_its[idx].func, ad);
		++idx;
	}
	elm_list_go(li);

	return li;
}

void datetime_cb(void *data, Evas_Object * obj, void *event_info)
{
	struct appdata *ad;

	Evas_Object *list;

	ad = (struct appdata *)data;
	if (!ad) return;

	list = _create_list_winset(ad);
	elm_naviframe_item_push(ad->nf, _("Datetime"), NULL, NULL, list, NULL);
}
