/*
 * Copyright (c) 2011 Samsung Electronics Co., Ltd All Rights Reserved
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

#include "main.h"

/*********************************************************
  Fastscroll
 ********************************************************/
typedef unsigned int uintptr_t;

#define NUM_OF_ITEMS 50
char *index_genlist_demo_names[] = {
	"AARON", "ABIS", "ACON", "ADAM", "AERIAL",
	"BALE", "BASIC", "BMIN", "BROD", "BTWIN",
	"CAT", "CBOX", "CCIN", "CEDRIC", "CELIA",
	"DOC", "DRUM", "DRANE", "DULL", "DVIW",
	"EASE", "EBIX", "ELIE", "EPIC", "ERIC",
	"FAME", "FAST", "FEN", "FIX", "FINE",
	"GAME", "GALEN", "GRACE", "GREG", "GRANNY",
	"HALEEM", "HARIS", "HENRIC", "HWITFORD", "HYATT",
	"IAN", "IDEN", "IESHA", "INGER", "ISHILD",
	"JAC", "JED", "JILL", "JON", "JUSTIN",
	NULL
};

static void _index_selected(void *data, Evas_Object *obj, void *event_info)
{
	elm_index_item_selected_set(event_info, EINA_FALSE);
}

static char *
_gli_text_get(void *data, Evas_Object *obj EINA_UNUSED, const char *part )
{
	int index = (int) data % NUM_OF_ITEMS;
	return strdup(index_genlist_demo_names[index]);
}

static void
_index_changed_cb(void *data EINA_UNUSED, Evas_Object *obj EINA_UNUSED, void *event_info EINA_UNUSED)
{
	// this is called on every change, no matter how often
	printf("changes called\n");
	elm_genlist_item_bring_in(elm_object_item_data_get(event_info),
		ELM_GENLIST_ITEM_SCROLLTO_TOP);
}

static void _index_level_up_cb(void *data , Evas_Object *obj , void *event_info)
{
	const Elm_Object_Item *it, *new_item;
	const Elm_Object_Item *index_obj;
	char buf[27], index1_letter;
	int i = 0, j = 0, index, flag = 0;
	int level = elm_index_item_level_get(obj);

	index_obj = elm_index_selected_item_get(obj, level - 1);
	it= (Elm_Object_Item*)elm_object_item_data_get(index_obj);
	if(it == NULL)
		return;
	index = (int)elm_object_item_data_get(it);
	index1_letter = index_genlist_demo_names[index][1];
	if(index1_letter) {
		char buf1[5] = {0,};
		snprintf(buf1, sizeof(buf1),"%c", index1_letter);

		elm_index_item_append(obj, buf1 , NULL, it);
		buf[i] = buf1[0];
		i++;

		while (it) {
			it = elm_genlist_item_next_get(it);
			if(it == NULL)
				break;

			new_item = elm_index_item_find(obj, it);
			if (new_item != NULL) break;

			index = (int)elm_object_item_data_get(it);
			index1_letter = index_genlist_demo_names[index][1];

			if (index1_letter != '\0') {
				for (j=0 ;j < i; j++) {
					if (buf[j] == index1_letter) {
						flag = 1;
						break;
					}
				}
				if (flag == 1) {
					flag = 0;
					continue;
				}
				buf[i] = index1_letter;
				i++;
				snprintf(buf1, sizeof(buf1),"%c", index1_letter);
				elm_index_item_append(obj, buf1 , NULL, it);
			}
		}
		elm_index_level_go(obj, 1);
	}
}

static void _gl_sel(void *data, Evas_Object *obj, void *event_info)
{
	elm_genlist_item_selected_set((Elm_Object_Item *)event_info, EINA_FALSE);
	return;
}

static Evas_Object* _create_fastscroll(Evas_Object* parent)
{
	Evas_Object *layout, *index, *gl;
	int j;
	int index1;
	Elm_Object_Item *glit;
	Elm_Genlist_Item_Class *itci = elm_genlist_item_class_new();
	//const char *style = "1text";
	layout = elm_layout_add(parent);
	elm_layout_theme_set(layout, "layout", "application", "fastscroll");
	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

	index = elm_index_add(parent);
	elm_layout_content_set(layout, "elm.swallow.fastscroll", index);
	evas_object_size_hint_weight_set(index, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(index, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_index_autohide_disabled_set(index, EINA_TRUE);

	gl = elm_genlist_add(parent);
	elm_genlist_mode_set(gl, ELM_LIST_COMPRESS);
	elm_scroller_policy_set(gl,ELM_SCROLLER_POLICY_OFF,ELM_SCROLLER_POLICY_OFF);
	evas_object_size_hint_weight_set(gl, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(gl, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_layout_content_set(layout, "elm.swallow.content", gl);

	itci->item_style = "1text";
	itci->func.text_get = _gli_text_get;
	itci->func.content_get  = NULL;
	itci->func.state_get = NULL;
	itci->func.del = NULL;

	j = 0;
	for (index1 = 0; index1 < NUM_OF_ITEMS; index1++) {
		glit = elm_genlist_item_append(gl, itci,
				(void *) index1,/* item data */
				NULL/* parent */,
				ELM_GENLIST_ITEM_NONE,
				_gl_sel/* func */, NULL/* func data */);

		if (index1 == 0 || !(index1 % 5)) {
			char buf[32];

			snprintf(buf, sizeof(buf), "%c", 'A' + j);
			elm_index_item_append(index, buf, NULL, glit);
			j += 1;
		}
	}
	elm_genlist_item_class_free(itci);
	evas_object_show(gl);
	elm_index_level_go(index, 0);
	elm_index_item_level_set(index,1);
	evas_object_smart_callback_add(index, "selected", _index_selected, NULL);
	evas_object_smart_callback_add(index, "level,up", _index_level_up_cb, NULL);
	evas_object_smart_callback_add(index, "changed", _index_changed_cb, NULL);
	evas_object_show(index);
	elm_naviframe_item_push(parent, "Fastscroll", NULL, NULL, layout, "empty");

	return index;
}

void _language_changed(void *data, Evas_Object *obj, void *event_info)
{
	int i = 0, j, len;
	char *str;
	char buf[PATH_MAX] = {0, };

	elm_index_item_clear(obj);

	str = dgettext("sys_string", "IDS_PB_BODY_ABCDEFGHIJKLMNOPQRSTUVWXYZ_M_INDEXLIST");
	len = strlen(str);

	while (1) {
		if (i >= len) break;

		j = i;
		eina_unicode_utf8_get_next(str, &i);
		snprintf(buf, i - j + 1, "%s", str + j);
		buf[i - j + 1] = 0;

		elm_index_item_append(obj, buf, NULL, NULL);
	}

	elm_index_level_go(obj, 0);
}

void
fastscroll_cb(void *data, Evas_Object *obj, void *event_info)
{
	struct appdata *ad;
	Evas_Object *index;

	ad = (struct appdata *) data;
	if (ad == NULL) return;

	index = _create_fastscroll(ad->nf);
	evas_object_smart_callback_add(index, "language,changed", _language_changed, index);
}
