/*
 * Copyright (c) 2011 Samsung Electronics Co., Ltd All Rights Reserved
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

#include "main.h"

/*********************************************************
  Naviframe
 ********************************************************/

static char *image_type[] = {
		"tw_ic_menu_detail_holo_dark.png",
		"tw_ic_menu_moreoverflow_holo_dark.png",
		"tw_ic_menu_selectall_holo_dark.png",
		"tw_ic_menu_delete_holo_dark.png",
		NULL
	};

static Evas_Object *layout_main = NULL;
static Evas_Object *conform = NULL;

static Evas_Object *_create_nocontent(Evas_Object *parent, const char *text)
{
	Evas_Object *layout, *noc, *sc;

	//Scroller
	sc = elm_scroller_add(parent);
	elm_scroller_bounce_set(sc, EINA_FALSE, EINA_TRUE);
	elm_scroller_policy_set(sc, ELM_SCROLLER_POLICY_OFF, ELM_SCROLLER_POLICY_AUTO);

	//NoContent
	noc = elm_layout_add(parent);
	if (!noc) return NULL;
	elm_layout_theme_set(noc, "layout", "nocontents", "full");
	elm_object_part_text_set(noc, "elm.text", text);

	//Layout
	layout = elm_layout_add(parent);
	if (!layout) {
		evas_object_del(noc);
		return NULL;
	}
	elm_layout_file_set(layout, ELM_DEMO_EDJ, "elmdemo-test/nocontents/full");
	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(layout, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_object_part_content_set(layout, "contents", noc);

	elm_object_content_set(sc, layout);
	return sc;
}

static Evas_Object *_create_title_btn(Evas_Object *parent, Evas_Smart_Cb func, void *data, int index)
{
	Evas_Object *ic;
	char buf[PATH_MAX];
	Evas_Object *btn = elm_button_add(parent);
	if (!btn) return NULL;
	elm_object_style_set(btn, "naviframe/title_icon");
	ic = elm_image_add(parent);
	snprintf(buf, sizeof(buf), "%s/%s", ICON_DIR, image_type[index]);
	elm_image_file_set(ic, buf, NULL);
	evas_object_size_hint_aspect_set(ic, EVAS_ASPECT_CONTROL_BOTH, 1, 1);
	elm_image_resizable_set(ic, EINA_TRUE, EINA_TRUE);
	elm_object_part_content_set(btn, "icon", ic);
	evas_object_smart_callback_add(btn, "clicked", func, data);
	return btn;
}

static void _quit_func(Evas_Object *nf)
{
	if (conform) {
		if (nf) {
			elm_object_content_unset(conform);
			evas_object_del(nf);
		}
		if (layout_main) {
			elm_object_content_set(conform, layout_main);
			evas_object_show(layout_main);
		}
		conform = NULL;
	}
}

static Eina_Bool _quit_cb(void *data, Elm_Object_Item *it)
{
	_quit_func(data);
	return EINA_FALSE;
}

static void _quit_button_cb(void *data, Evas_Object *obj, void *event_info)
{
	_quit_func(data);
}

static void _fourth_page(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *content, *btn;
	Evas_Object *nf = (Evas_Object *) data;
	Elm_Object_Item *navi_it;

	if (nf == NULL) return;

	content = _create_nocontent(nf, "Naviframe Demo<br>Page 4");
	btn = _create_title_btn(nf, _quit_button_cb, nf, 3);

	navi_it = elm_naviframe_item_push(nf, "Naviframe More Option", NULL, NULL, content, NULL);
	elm_object_item_part_content_set(navi_it, "title_right_btn", btn);
}

static void _third_page(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *content, *btn;
	Evas_Object *nf = (Evas_Object *) data;
	Elm_Object_Item *navi_it;

	if (nf == NULL) return;

	content = _create_nocontent(nf, "Naviframe Demo<br>Page 3");
	btn = _create_title_btn(nf, _fourth_page, nf, 2);

	navi_it = elm_naviframe_item_push(nf, "Naviframe More Option", NULL, NULL, content, NULL);
	elm_object_item_part_content_set(navi_it, "title_right_btn", btn);
}

static void _second_page(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *content, *btn;
	Evas_Object *nf = (Evas_Object *) data;
	Elm_Object_Item *navi_it;

	if (nf == NULL) return;

	content = _create_nocontent(nf, "Naviframe Demo<br>Page 2");
	btn = _create_title_btn(nf, _third_page, nf, 1);

	navi_it = elm_naviframe_item_push(nf, "Naviframe More Option", NULL, NULL, content, NULL);
	elm_object_item_part_content_set(navi_it, "title_right_btn", btn);
}

static void _first_page(Evas_Object *nf)
{
	Evas_Object *content, *btn, *box;
	Elm_Object_Item *navi_it;

	if (nf == NULL) return;

	box = elm_box_add(nf);
	evas_object_size_hint_weight_set(box, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(box, EVAS_HINT_FILL, EVAS_HINT_FILL);

	content = _create_nocontent(box, "Naviframe Demo<br>Page 1");
	evas_object_size_hint_weight_set(content, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(content, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_show(content);
	elm_box_pack_end(box, content);
	evas_object_show(box);

	btn = _create_title_btn(nf, _second_page, nf, 0);

	navi_it = elm_naviframe_item_push(nf, "Naviframe More Option", NULL, NULL, box, NULL);
	elm_object_item_part_content_set(navi_it, "title_right_btn", btn);
	elm_naviframe_item_pop_cb_set(navi_it, _quit_cb, nf);
}

void naviframe_cb(void *data, Evas_Object *obj, void *event_info)
{
	struct appdata *ad;
	Evas_Object *nf;

	ad = (struct appdata *) data;
	if (ad == NULL) return;

	layout_main = ad->layout_main;
	elm_object_content_unset(ad->conform);
	evas_object_hide(ad->layout_main);

	nf = elm_naviframe_add(ad->conform);
	eext_object_event_callback_add(nf, EEXT_CALLBACK_BACK, eext_naviframe_back_cb, NULL);
	eext_object_event_callback_add(nf, EEXT_CALLBACK_MORE, eext_naviframe_more_cb, NULL);
	elm_object_content_set(ad->conform, nf);
	evas_object_show(nf);

	conform = ad->conform;
	_first_page(nf);
}
