/*
 * Copyright (c) 2011 Samsung Electronics Co., Ltd All Rights Reserved
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

#include "main.h"

/*********************************************************
  popup
 ********************************************************/
static Evas_Object *list = NULL;
Evas_Object *top = NULL;
static int state_index = 0;

typedef struct _Item_Data
{
	int index;
	Evas_Object *genlist;
	Elm_Object_Item *item;
} Item_Data;

static char *Items[] = {
	"Benny Benassi", "Main Item 2", "Main Item 3",
	"Main Item 4", "Main Item 5", "Main Item 6",
	"Main Item 7", "Main Item 8"
};
static char *Sub_Items[] = {
	"08282828282", "Sub Item 2", "Sub Item 3",
	"Sub Item 4", "Sub Item 5", "Sub Item 6",
	"Sub Item 7", "Sub Item 8"
};

static void _list_click(void *data, Evas_Object *obj, void *event_info)
{
	Elm_Object_Item *it = (Elm_Object_Item *) elm_list_selected_item_get(obj);
	if (!it) return;
	elm_list_item_selected_set(it, EINA_FALSE);
}

static void _timeout_cb(void *data, Evas_Object *obj, void *event_info)
{
	evas_object_del(obj);
}

static void _response_cb(void *data, Evas_Object *obj, void *event_info)
{
	evas_object_del(data);
}

static void _list_item_click(Evas_Object *obj, int index)
{
	printf("Index of the list=%d\n", index);
	evas_object_del(obj);
}

static void _block_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	evas_object_del(obj);
}

static void _popup_text_cb(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *popup;
	struct appdata *ad;

	ad = (struct appdata *) data;

	popup = elm_popup_add(ad->win_main);
	evas_object_size_hint_weight_set(popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_popup_align_set(popup, ELM_NOTIFY_ALIGN_FILL, 1.0);
	eext_object_event_callback_add(popup, EEXT_CALLBACK_BACK, eext_popup_back_cb, NULL);
	evas_object_smart_callback_add(popup, "timeout", _timeout_cb, NULL);
	elm_object_text_set(popup, "This has only texts. This is set by object "
			"text set");
	evas_object_show(popup);
}

static void _popup_scrollable_text_cb(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *popup;
	struct appdata *ad;

	ad = (struct appdata *) data;

	popup = elm_popup_add(ad->win_main);
	evas_object_size_hint_weight_set(popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_popup_align_set(popup, ELM_NOTIFY_ALIGN_FILL, 1.0);
	eext_object_event_callback_add(popup, EEXT_CALLBACK_BACK, eext_popup_back_cb, NULL);
	evas_object_smart_callback_add(popup, "timeout", _timeout_cb, NULL);

	elm_object_text_set(popup, "This is scrollable popup text."
		"This part is made by adding long text in popup. Popup internally added "
		"scroller to this layout when size of text is greater than total popup "
		"height. This popup does not have buttons.");

	evas_object_show(popup);
}

static void _popup_title_text_cb(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *popup;
	struct appdata *ad;
	ad = (struct appdata *) data;

	popup = elm_popup_add(ad->win_main);
	evas_object_size_hint_weight_set(popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_popup_align_set(popup, ELM_NOTIFY_ALIGN_FILL, 1.0);
	elm_object_part_text_set(popup, "title,text", "Title");
	elm_object_text_set(popup,"This Popup has title area and text and 3 second "
			"timeout");
	eext_object_event_callback_add(popup, EEXT_CALLBACK_BACK, eext_popup_back_cb, NULL);
	elm_popup_timeout_set(popup, 3.0);
	evas_object_smart_callback_add(popup, "timeout", _timeout_cb, NULL);
	evas_object_show(popup);
}

static void _popup_text_1button_cb(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *popup;
	Evas_Object *btn;
	struct appdata *ad = (struct appdata *) data;

	popup = elm_popup_add(ad->win_main);
	evas_object_size_hint_weight_set(popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_popup_align_set(popup, ELM_NOTIFY_ALIGN_FILL, 1.0);
	eext_object_event_callback_add(popup, EEXT_CALLBACK_BACK, eext_popup_back_cb, NULL);

	elm_object_text_set(popup, "This is scrollable popup text."
		"This part is made by adding long text in popup. Popup internally added "
		"scroller to this layout when size of text is greater than total popup "
		"height. This has one button in action area and does not have title area");

	btn = elm_button_add(popup);
	elm_object_style_set(btn, "popup");
	evas_object_size_hint_weight_set(btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_text_set(btn, "Cancel");
	elm_object_part_content_set(popup, "button1", btn);
	evas_object_smart_callback_add(btn, "clicked", _response_cb, popup);

	evas_object_show(popup);
}

static void _popup_title_text_1button_cb(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *popup;
	Evas_Object *btn;
	struct appdata *ad = (struct appdata *) data;

	popup = elm_popup_add(ad->win_main);
	evas_object_size_hint_weight_set(popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_popup_align_set(popup, ELM_NOTIFY_ALIGN_FILL, 1.0);
	elm_object_part_text_set(popup, "title,text", "Title");
	eext_object_event_callback_add(popup, EEXT_CALLBACK_BACK, eext_popup_back_cb, NULL);

	elm_object_text_set(popup, "This is scrollable popup text."
		"This part is made by adding long text in popup. Popup internally added "
		"scroller to this layout when size of text is greater than total popup "
		"height. This has one button in action area and title text in title area");

	btn = elm_button_add(popup);
	elm_object_style_set(btn, "popup");
	evas_object_size_hint_weight_set(btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_text_set(btn, "Cancel");
	elm_object_part_content_set(popup, "button1", btn);
	evas_object_smart_callback_add(btn, "clicked", _response_cb, popup);
	evas_object_show(popup);
}

static void _popup_title_text_2button_cb(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *popup;
	Evas_Object *btn;
	struct appdata *ad = (struct appdata *) data;

	popup = elm_popup_add(ad->win_main);
	evas_object_size_hint_weight_set(popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_popup_align_set(popup, ELM_NOTIFY_ALIGN_FILL, 1.0);
	elm_object_part_text_set(popup, "title,text", "Title");
	eext_object_event_callback_add(popup, EEXT_CALLBACK_BACK, eext_popup_back_cb, NULL);

	elm_object_text_set(popup, "This is scrollable popup text."
		"This part is made by adding long text in popup. Popup internally added "
		"scroller to this layout when size of text is greater than total popup "
		"height. This has two buttons in action area and title text in title area");

	btn = elm_button_add(popup);
	elm_object_style_set(btn, "popup");
	evas_object_size_hint_weight_set(btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_text_set(btn, "Cancel");
	elm_object_part_content_set(popup, "button1", btn);
	evas_object_smart_callback_add(btn, "clicked", _response_cb, popup);

	btn = elm_button_add(popup);
	elm_object_style_set(btn, "popup");
	evas_object_size_hint_weight_set(btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_text_set(btn, "OK");
	elm_object_part_content_set(popup, "button2", btn);
	evas_object_smart_callback_add(btn, "clicked", _response_cb, popup);

	evas_object_show(popup);
}

static void _radio_cb(void *data, Evas_Object *obj, void *event_info)
{
	int x = 0;
	printf("Selected Radio:%d\n", (int)(long)data);
	x = elm_radio_value_get(obj);
	printf("selected value for the group:%d\n", x);
}

static void _gl_sel(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *popup = data;
	int index = 0;
	Elm_Object_Item *item = (Elm_Object_Item *)event_info;
	if (item) {
		index = (int)elm_object_item_data_get(item);
		fprintf(stdout, "selected text %s\n",Items[index]);
	}
	_list_item_click(popup, index);
}

static void _gl_radio_list_sel(void *data, Evas_Object *obj, void *event_info)
{
	Elm_Object_Item *item = (Elm_Object_Item *) event_info;

	if (item) {
		Item_Data *id = elm_object_item_data_get(item);
		int index = id->index;
		Evas_Object *radio = elm_object_item_part_content_get(item, "elm.icon");
		state_index = index;
		elm_radio_value_set(radio, state_index);
	}
}

static char *_gl_text_get(void *data, Evas_Object *obj, const char *part)
{
	int index = (int) data;

	if (!strcmp(part, "elm.text.sub"))
		return strdup(Sub_Items[index]);
	else
		return strdup(Items[index]);
}

static Evas_Object *_gl_content_get(void *data, Evas_Object *obj, const char *part)
{
	Evas_Object *icon;
	if (!strcmp(part, "elm.icon")) {
		icon = elm_image_add(obj);
		elm_image_file_set(icon, ICON_DIR"/g1.png", NULL);
		evas_object_size_hint_aspect_set(icon, EVAS_ASPECT_CONTROL_VERTICAL, 1, 1);
		return icon;
	}
	else {
		return NULL;
	}
}

static char *_gl_radio_list_text_get(void *data, Evas_Object *obj, const char *part)
{
	Item_Data *id = data;
	int index = id->index;

	if (!strcmp(part, "elm.text.sub"))
		return strdup(Sub_Items[index]);
	else
		return strdup(Items[index]);
}

static Evas_Object *_gl_radio_list_content_get(void *data, Evas_Object *obj, const char *part)
{
	Item_Data *id = data;
	int index = id->index;
	Evas_Object *radio;
	Evas_Object *radio_main = evas_object_data_get(obj, "radio_main");

	if (!strcmp(part, "elm.icon")) {
		radio = elm_radio_add(obj);
		elm_radio_state_value_set(radio, index);
		elm_radio_group_add(radio, radio_main);
		if (index == state_index)
			elm_radio_value_set(radio, state_index);
		evas_object_size_hint_weight_set(radio, EVAS_HINT_EXPAND,
			EVAS_HINT_EXPAND);
		evas_object_size_hint_align_set(radio, EVAS_HINT_FILL, EVAS_HINT_FILL);
		evas_object_propagate_events_set(radio, EINA_FALSE);
		evas_object_smart_callback_add(radio, "changed", _radio_cb, id);

		return radio;
	}

	return NULL;
}

static void _popup_title_list_cb(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *popup, *genlist;
	unsigned int index;
	Elm_Object_Item *item;
	char buf[4096];
	struct appdata *ad = data;

	popup = elm_popup_add(ad->win_main);
	evas_object_size_hint_weight_set(popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_popup_align_set(popup, ELM_NOTIFY_ALIGN_FILL, 1.0);
	elm_object_part_text_set(popup, "title,text", "Title");
	eext_object_event_callback_add(popup, EEXT_CALLBACK_BACK, eext_popup_back_cb, NULL);

	Elm_Genlist_Item_Class *itc;
	itc = elm_genlist_item_class_new();
	itc->item_style = "1text.1icon";
	itc->func.text_get = _gl_text_get;
	itc->func.content_get = _gl_content_get;

	genlist = elm_genlist_add(popup);
	elm_object_style_set(genlist, "popup");
	elm_genlist_homogeneous_set(genlist, EINA_TRUE);
	elm_scroller_content_min_limit(genlist, EINA_FALSE, EINA_TRUE);

	for (index = 0; index < 7; index++) {
		snprintf(buf, sizeof(buf), "%u Popup List Item", index+1);
		item = elm_genlist_item_append(genlist, itc, NULL, NULL, ELM_GENLIST_ITEM_NONE, _gl_sel, NULL);
		if (index == 1)
			elm_object_item_disabled_set(item ,EINA_TRUE);
		if (index == 6)
			elm_object_item_signal_emit(item, "elm,state,bottomline,hide", "");
	}
	elm_object_content_set(popup, genlist);
	elm_genlist_item_class_free(itc);

	evas_object_show(popup);
}

static void _popup_list_cb(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *popup, *genlist;
	unsigned int index;
	Elm_Object_Item *item;
	char buf[4096];
	struct appdata *ad = data;

	popup = elm_popup_add(ad->win_main);
	evas_object_size_hint_weight_set(popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_popup_align_set(popup, ELM_NOTIFY_ALIGN_FILL, 1.0);
	eext_object_event_callback_add(popup, EEXT_CALLBACK_BACK, eext_popup_back_cb, NULL);

	Elm_Genlist_Item_Class *itc;
	itc = elm_genlist_item_class_new();
	itc->item_style = "1text.1icon";
	itc->func.text_get = _gl_text_get;
	itc->func.content_get = _gl_content_get;

	genlist = elm_genlist_add(popup);
	elm_object_style_set(genlist, "popup");
	elm_genlist_homogeneous_set(genlist, EINA_TRUE);
	elm_scroller_content_min_limit(genlist, EINA_FALSE, EINA_TRUE);

	for (index = 0; index < 7; index++) {
		snprintf(buf, sizeof(buf), "%u Popup List Item", index+1);
		item = elm_genlist_item_append(genlist, itc, NULL, NULL, ELM_GENLIST_ITEM_NONE, _gl_sel, NULL);
		if (index == 1)
			elm_object_item_disabled_set(item ,EINA_TRUE);
		if (index == 6)
			elm_object_item_signal_emit(item, "elm,state,bottomline,hide", "");
	}
	elm_object_content_set(popup, genlist);
	elm_genlist_item_class_free(itc);

	evas_object_show(popup);
}

static void _popup_title_list_radio_cb(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *popup, *genlist;
	unsigned int index;
	Elm_Object_Item *item;
	char buf[4096];
	struct appdata *ad = data;

	popup = elm_popup_add(ad->win_main);
	evas_object_size_hint_weight_set(popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_popup_align_set(popup, ELM_NOTIFY_ALIGN_FILL, 1.0);
	elm_object_part_text_set(popup, "title,text", "Title");
	eext_object_event_callback_add(popup, EEXT_CALLBACK_BACK, eext_popup_back_cb, NULL);

	Elm_Genlist_Item_Class *itc;
	itc = elm_genlist_item_class_new();
	itc->item_style = "1text.1icon.1";
	itc->func.text_get = _gl_radio_list_text_get;
	itc->func.content_get = _gl_radio_list_content_get;

	genlist = elm_genlist_add(popup);
	elm_object_style_set(genlist, "popup");
	elm_genlist_mode_set(genlist, ELM_LIST_COMPRESS);
	elm_genlist_homogeneous_set(genlist, EINA_TRUE);
	elm_scroller_content_min_limit(genlist, EINA_FALSE, EINA_TRUE);

	Evas_Object *radio_main = elm_radio_add(genlist);
	elm_radio_state_value_set(radio_main, 0);
	elm_radio_value_set(radio_main, 0);
	evas_object_data_set(genlist, "radio_main", radio_main);

	for (index = 0; index < 7; index++) {
		Item_Data *id = calloc(sizeof(Item_Data), 1);
		snprintf(buf, sizeof(buf), "%u Popup List Item", index+1);
		id->index  = index;
		item = elm_genlist_item_append(genlist, itc, id, NULL, ELM_GENLIST_ITEM_NONE, _gl_radio_list_sel, NULL);
		id->item = item;
		id->genlist = genlist;
		if (index == 1)
			elm_object_item_disabled_set(item ,EINA_TRUE);
		if (index == 6)
			elm_object_item_signal_emit(item, "elm,state,bottomline,hide", "");
	}
	elm_object_content_set(popup, genlist);
	evas_object_show(genlist);
	elm_genlist_item_class_free(itc);

	evas_object_show(popup);
}

static void _popup_title_text_check_button_cb(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *popup, *label, *layout_inner, *check;
	Evas_Object *btn1,*btn2;
	struct appdata *ad = (struct appdata *) data;

	popup = elm_popup_add(ad->win_main);
	evas_object_size_hint_weight_set(popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_popup_align_set(popup, ELM_NOTIFY_ALIGN_FILL, 1.0);
	elm_object_part_text_set(popup, "title,text", "Popup Title");

	layout_inner = elm_layout_add(popup);
	elm_layout_file_set(layout_inner, ELM_DEMO_EDJ, "popup_checkview_internal");
	evas_object_size_hint_weight_set(layout_inner, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_content_set(popup, layout_inner);

	label = elm_label_add(layout_inner);
	elm_object_style_set(label, "popup/default");
	elm_label_line_wrap_set(label, ELM_WRAP_MIXED);
	elm_object_text_set(label, "Will be saved, and sound, only on the Gear.");
	evas_object_size_hint_weight_set(label, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(label, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_object_part_content_set(layout_inner, "label", label);

	check = elm_check_add(layout_inner);
	elm_object_style_set(check, "popup");
	elm_object_text_set(check, _("Don't repeat again Don't repeat again ?"));
	elm_object_part_content_set(layout_inner, "elm.swallow.end", check);

	btn1 = elm_button_add(popup);
	elm_object_style_set(btn1, "popup");
	elm_object_text_set(btn1, "Cancel");
	elm_object_part_content_set(popup, "button1", btn1);
	evas_object_smart_callback_add(btn1, "clicked", _response_cb, popup);

	btn2 = elm_button_add(popup);
	elm_object_style_set(btn2, "popup");
	elm_object_text_set(btn2, "OK");
	elm_object_part_content_set(popup, "button2", btn2);
	evas_object_smart_callback_add(btn2, "clicked", _response_cb, popup);

	eext_object_event_callback_add(popup, EEXT_CALLBACK_BACK, eext_popup_back_cb, ad);

	evas_object_show(popup);
}

static void _popup_title_list_button_cb(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *popup, *btn, *genlist;
	unsigned int index;
	Elm_Object_Item *item;
	char buf[4096];
	struct appdata *ad = data;

	popup = elm_popup_add(ad->win_main);
	evas_object_size_hint_weight_set(popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_popup_align_set(popup, ELM_NOTIFY_ALIGN_FILL, 1.0);
	elm_object_part_text_set(popup, "title,text", "Title");
	eext_object_event_callback_add(popup, EEXT_CALLBACK_BACK, eext_popup_back_cb, NULL);

	Elm_Genlist_Item_Class *itc;
	itc = elm_genlist_item_class_new();
	itc->item_style = "1text";
	itc->func.text_get = _gl_text_get;
	itc->func.content_get = _gl_content_get;

	genlist = elm_genlist_add(popup);
	elm_object_style_set(genlist, "popup");
	elm_genlist_homogeneous_set(genlist, EINA_TRUE);
	elm_scroller_content_min_limit(genlist, EINA_FALSE, EINA_TRUE);

	for (index = 0; index < 7; index++) {
		snprintf(buf, sizeof(buf), "%u Popup List Item", index+1);
		item = elm_genlist_item_append(genlist, itc, NULL, NULL, ELM_GENLIST_ITEM_NONE, _gl_sel, NULL);
		if (index == 1)
			elm_object_item_disabled_set(item ,EINA_TRUE);
		if (index == 6)
			elm_object_item_signal_emit(item, "elm,state,bottomline,hide", "");
	}
	elm_object_content_set(popup, genlist);
	elm_genlist_item_class_free(itc);

	btn = elm_button_add(popup);
	elm_object_style_set(btn, "popup");
	evas_object_size_hint_weight_set(btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_text_set(btn, "Cancel");
	elm_object_part_content_set(popup, "button1", btn);
	evas_object_smart_callback_add(btn, "clicked", _response_cb, popup);

	evas_object_show(popup);
}

static void _popup_toast_cb(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *popup;
	struct appdata *ad = data;

	popup = elm_popup_add(ad->win_main);
	elm_object_style_set(popup, "toast");
	elm_popup_orient_set(popup, ELM_POPUP_ORIENT_BOTTOM);
	evas_object_size_hint_weight_set(popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	//elm_popup_align_set(popup, ELM_NOTIFY_ALIGN_FILL, 1.0);
	eext_object_event_callback_add(popup, EEXT_CALLBACK_BACK, eext_popup_back_cb, NULL);
	elm_object_part_text_set(popup,"elm.text", "Saving Contacts to sim on sams");

	evas_object_smart_callback_add(popup, "block,clicked", _block_clicked_cb, NULL);
	evas_object_smart_callback_add(popup, "timeout", _timeout_cb, NULL);

	evas_object_show(popup);
}

void popup_cb(void *data, Evas_Object *obj, void *event_info)
{
	struct appdata *ad = data;
	if (ad == NULL) return;

	list = elm_list_add(ad->nf);
	elm_list_mode_set(list, ELM_LIST_COMPRESS);
	evas_object_smart_callback_add(list, "selected", _list_click, NULL);
	elm_list_item_append(list, "Text", NULL, NULL,
			_popup_text_cb, ad);
	elm_list_item_append(list, "Text scrollable", NULL, NULL,
			_popup_scrollable_text_cb, ad);
	elm_list_item_append(list, "Title text", NULL, NULL,
			_popup_title_text_cb, ad);
	elm_list_item_append(list, "Text 1button", NULL, NULL,
			_popup_text_1button_cb, ad);
	elm_list_item_append(list, "Title text 1button", NULL, NULL,
			_popup_title_text_1button_cb, ad);
	elm_list_item_append(list, "Title text 2button", NULL, NULL,
			_popup_title_text_2button_cb, ad);
	elm_list_item_append(list, "Title list", NULL, NULL,
			_popup_title_list_cb, ad);
	elm_list_item_append(list, "List", NULL, NULL,
			_popup_list_cb, ad);
	elm_list_item_append(list, "Title list radio", NULL, NULL,
			_popup_title_list_radio_cb, ad);
	elm_list_item_append(list, "Title list button", NULL, NULL,
			_popup_title_list_button_cb, ad);
	elm_list_item_append(list, "Title text check button", NULL, NULL,
			_popup_title_text_check_button_cb, ad);
	elm_list_item_append(list, "Toast", NULL, NULL,
			_popup_toast_cb, ad);
	elm_list_go(list);

	elm_naviframe_item_push(ad->nf, _("Popup"), NULL, NULL, list, NULL);
}
